from __future__ import absolute_import

from .source import Source
from .c_tokenizer import CTokenizer
from .cpp_tokenizer import CppTokenizer
from .java_tokenizer import JavaTokenizer
from .python_tokenizer import PythonTokenizer

def tokenize_str(source_str, lang=None):
    src = Source.from_str(source_str, lang)
    return src.tokenize() 

def tokenize_file(filepath, lang=None):
    src = Source.from_file(filepath, lang)
    return src.tokenize()

    


import enum

class TokenType(enum.Enum):
    KEYWORD = 0 
    IDENTIFIER = 1 
    CONSTANT = 2 # numbers and characters
    STRING = 3
    SPECIAL_SYMBOL = 4
    OPERATOR = 5
    COMMENT_SYMBOL = 6
    OTHER = 7

class Token():
    def __init__(self, token_value, token_type, line, column):
        self.token_value = token_value
        self.token_type = token_type
        self.line = line
        self.column = column
    
    @property
    def position(self):
        return (self.line, self.column)

    def __str__(self):
        return '({}, {}, {})'.\
            format(self.token_value, self.token_type, self.position)

    def __repr__(self):
        return self.__str__()
    
    




from __future__ import absolute_import 

from sctokenizer.assets.c_keywords import *
from sctokenizer.assets.cpp_keywords import *
from sctokenizer.assets.java_keywords import *
from sctokenizer.assets.python_keywords import *



from __future__ import absolute_import

from sctokenizer.tokenizer import Tokenizer, TokenizerState
from sctokenizer.assets.php_keywords import php_keyword_set
from sctokenizer.assets.php_operators import php_operator_set
from sctokenizer.token import TokenType, Token

class PhpTokenizer(Tokenizer):
    
    def __init__(self):
        super().__init__()
        self.keyword_set = php_keyword_set
        self.operator_set = php_operator_set

    def tokenize(self, source_str):
        if len(source_str) < 1:
            return []
        len_lines = [len(x) for x in source_str.split('\n')]
        tokens = []
        state = TokenizerState.REGULAR
        pending = ''
        first_no_space = ''
        first_no_space_in_word = ''
        first_char_in_string = ''
        cur = ''
        prev = ''
        i = 0
        t = 0
        while i < len(source_str):
            prev = cur
            cur = source_str[i]
            if i < len(source_str) - 1:
                next = source_str[i+1]
            else:
                next = ''

            if prev == '\n':
                first_no_space = ''
                first_no_space_in_word = ''
                self.linenumber += 1
                t += 1
            if cur == '\r':
                if next == '\n':
                    i += 1
                    continue
                else: # Not sure about this part
                    self.linenumber += 1
                    t += 1
                    cur = '\n'
            if cur != ' ' and cur != '\t':
                if first_no_space == '':
                    first_no_space = cur
                if first_no_space_in_word == '':
                    first_no_space_in_word = cur
                    self.colnumber = i
            if state == TokenizerState.IN_COMMENT:
                # Check end of block comment
                if cur == '*':
                    if next == '/':
                        self.colnumber = i
                        self.add_pending(tokens, '*/', TokenType.COMMENT_SYMBOL, len_lines, t)
                        i += 1
                        state = TokenizerState.REGULAR
                        continue
            elif state == TokenizerState.IN_LINECOMMENT:
                # Check end of line comment
                if cur == '\n':
                    state = TokenizerState.REGULAR
            elif state == TokenizerState.IN_STRING:
                # Check end of string
                if cur == first_char_in_string and prev != '\\':
                    first_char_in_string = ''
                    state = TokenizerState.REGULAR
                    self.add_pending(tokens, pending, TokenType.STRING, len_lines, t)
                    pending = ''
                    first_no_space_in_word = ''
                    self.colnumber = i
                    self.add_pending(tokens, cur, TokenType.SPECIAL_SYMBOL, len_lines, t)
                else:
                    pending += cur
            elif state == TokenizerState.IN_NUMBER:
                if (cur >= '0' and cur <= '9') or \
                    cur == '.' or cur == 'E' or cur == 'e':
                    pending += cur
                    i += 1
                    continue
                if (cur == '-' or cur == '+') and \
                    (prev == 'E' or prev == 'e'):
                    pending += cur
                    i += 1
                    continue
                self.add_pending(tokens, pending, TokenType.CONSTANT, len_lines, t)
                first_no_space_in_word = cur
                pending = cur
                self.colnumber = i
                state = TokenizerState.REGULAR 
            elif state == TokenizerState.REGULAR:
                if pending in self.operator_set:
                    if (pending + cur) in self.operator_set:
                        pending += cur
                        i += 1
                        continue
                    else:
                        self.add_pending(tokens, pending, TokenType.OPERATOR, len_lines, t)
                        pending = ''
                        first_no_space_in_word = cur
                        self.colnumber = i

                if cur == '/':
                    if next == '*': # Begin block comments
                        state = TokenizerState.IN_COMMENT
                        self.add_pending(tokens, pending, TokenType.IDENTIFIER, len_lines, t)
                        pending = ''
                        first_no_space_in_word = ''
                        self.colnumber = i
                        self.add_pending(tokens, '/*', TokenType.COMMENT_SYMBOL, len_lines, t)
                        i += 1
                        continue
                    if next == '/': # Begin line comment
                        state = TokenizerState.IN_LINECOMMENT
                        self.add_pending(tokens, pending, TokenType.IDENTIFIER, len_lines, t)
                        pending = ''
                        first_no_space_in_word = ''
                        self.colnumber = i
                        self.add_pending(tokens, '//', TokenType.COMMENT_SYMBOL, len_lines, t)
                        i += 1
                        continue
                elif cur == '"' or cur == "'":
                    if first_char_in_string == '':
                        first_char_in_string = cur
                    state = TokenizerState.IN_STRING
                    self.add_pending(tokens, pending, TokenType.IDENTIFIER, len_lines, t)
                    pending = ''
                    first_no_space_in_word = ''
                    self.colnumber = i
                    self.add_pending(tokens, cur, TokenType.SPECIAL_SYMBOL, len_lines, t)
                elif cur == '#':
                    # Begin line comment
                    state = TokenizerState.IN_LINECOMMENT
                    self.add_pending(tokens, pending, TokenType.IDENTIFIER, len_lines, t)
                    pending = ''
                    first_no_space_in_word = ''
                    self.colnumber = i
                    self.add_pending(tokens, '#', TokenType.COMMENT_SYMBOL, len_lines, t)
                    i += 1
                    continue
                elif cur >= '0' and cur <= '9':
                    if first_no_space_in_word == cur:
                        state = TokenizerState.IN_NUMBER
                        self.add_pending(tokens, pending, TokenType.IDENTIFIER, len_lines, t)
                        # first_no_space_in_word = ''
                        pending = cur
                    else:
                        pending += cur
                elif self.is_alpha(cur):
                    pending += cur
                elif cur in self.operator_set: # cur = + - * / , ...
                    self.add_pending(tokens, pending, TokenType.IDENTIFIER, len_lines, t)
                    pending = cur
                    first_no_space_in_word = cur
                    self.colnumber = i
                elif cur == '$':
                    pending += cur
                else: # cur = ;, ', space
                    self.add_pending(tokens, pending, TokenType.IDENTIFIER, len_lines, t)
                    pending = ''
                    first_no_space_in_word = ''
                    if cur > ' ': 
                        self.colnumber = i
                        self.add_pending(tokens, cur, TokenType.SPECIAL_SYMBOL, len_lines, t)
            i += 1
        # End of the program
        # This need to be fixed in the future
        if len(cur) > 1 or self.is_alpha(cur):
            self.add_pending(tokens, pending, TokenType.IDENTIFIER, len_lines, t) 
        else:
            self.add_pending(tokens, pending, TokenType.SPECIAL_SYMBOL, len_lines, t)
        return tokens




from __future__ import absolute_import

from sctokenizer.c_tokenizer import CTokenizer
from sctokenizer.cpp_tokenizer import CppTokenizer
from sctokenizer.java_tokenizer import JavaTokenizer
from sctokenizer.python_tokenizer import PythonTokenizer
from sctokenizer.php_tokenizer import PhpTokenizer

import os
import enum

LANG_MAP = {
    'cc': 'cpp',
    'py': 'python',
}

def check_language(lang):
    if lang in LANG_MAP:
        return LANG_MAP[lang]
    return lang

class SourceState(enum.Enum):
    INIT = 0
    UNTOKENIZED = 1
    TOKENIZED = 2

class Source():
    def __init__(self, source_str, lang=None, name=None):
        self.__state = SourceState.INIT

        self.source_str = source_str
        if lang is None:
            self.lang = self.detect_language(self.source_str)
        else:
            self.lang = check_language(lang)
        self.name = name
        self.tokens = None
    
    @classmethod
    def from_file(cls, filepath, lang=None, name=None):
        """
            return the Source object
            :rtype: Source
        """
        with open(filepath) as f:
            source_str = f.read()
        if lang is None:
            ext = os.path.splitext(filepath)[1][1:]
            lang = LANG_MAP[ext]
        if name is None:
            name = filepath
        return Source(source_str, lang, name)

    @classmethod
    def from_str(cls, source_str, lang=None, name=None):
        """
            return the Source object
            :rtype: Source
        """
        if lang is None:
            lang = cls.detect_language(source_str)
        return Source(source_str, lang, name)

    def get_language(self):
        return self.lang

    def get_name(self):
        return self.name

    def get_source_str(self):
        return self.source_str

    def tokenize(self):
        if self.__state == SourceState.TOKENIZED:
            return self.tokens

        if self.lang == 'c':
            c_tokenizer = CTokenizer()
            self.tokens = c_tokenizer.tokenize(self.source_str)
        elif self.lang == 'cpp':
            cpp_tokenizer = CppTokenizer()
            self.tokens = cpp_tokenizer.tokenize(self.source_str)
        elif self.lang == 'java':
            java_tokenizer = JavaTokenizer()
            self.tokens = java_tokenizer.tokenize(self.source_str)
        elif self.lang == 'python':
            python_tokenizer = PythonTokenizer()
            self.tokens = python_tokenizer.tokenize(self.source_str)
        elif self.lang == 'php':
            php_tokenizer = PhpTokenizer()
            self.tokens = php_tokenizer.tokenize(self.source_str)
        else:
            raise ValueError("Upsupported language")

        self.__state = SourceState.TOKENIZED
        return self.tokens

    @classmethod
    def detect_language(cls, source_str):
        """
            detect languge of source code
            :rtype: str
        """
        return 'cpp'




c_keyword_set = {
    'auto',
    'break',
    'case',
    'char',
    'const',
    'continue',
    'default',
    'define',
    'do',
    'double',
    'elif',
    'else',
    'endif',
    'enum',
    'error',
    'extern',
    'float',
    'for',
    'goto',
    'if',
    'ifdef',
    'ifndef',
    'include',
    'inline',
    'int',
    'line',
    'long',
    'noalias',
    'pragma',
    'register',
    'restrict',
    'return',
    'short',
    'signed',
    'sizeof',
    'static',
    'struct',
    'switch',
    'typedef',
    'undef',
    'union',
    'unsigned',
    'void',
    'volatile',
    'while'
}


from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(name='sctokenizer',
      description='A Source Code Tokenizer',
      author='Ngoc Bui',
      long_description=long_description,
      long_description_content_type="text/markdown",
      author_email='ngocjr7@gmail.com',
      version='0.0.4', 
      packages=find_packages(),
      python_requires='>=3.6')
